package ex.hm;

import bayonet.math.SpecialFunctions;
import blang.core.RealVar;
import blang.types.Index;
import blang.types.Plate;
import blang.types.Plated;
import java.util.Collection;
import java.util.LinkedHashMap;
import java.util.Map;
import org.eclipse.xtend.lib.annotations.Data;
import org.eclipse.xtext.xbase.lib.Pure;

@SuppressWarnings("all")
public class Utils {
  @Data
  public static class Logistic implements RealVar {
    private final RealVar beta0;
    
    private final RealVar beta1;
    
    private final int x;
    
    @Override
    public double doubleValue() {
      double _doubleValue = this.beta0.doubleValue();
      double _doubleValue_1 = this.beta1.doubleValue();
      double _multiply = (_doubleValue_1 * this.x);
      double _plus = (_doubleValue + _multiply);
      return SpecialFunctions.logistic(_plus);
    }
    
    @Override
    public String toString() {
      double _doubleValue = this.doubleValue();
      return ("" + Double.valueOf(_doubleValue));
    }
    
    public Logistic(final RealVar beta0, final RealVar beta1, final int x) {
      super();
      this.beta0 = beta0;
      this.beta1 = beta1;
      this.x = x;
    }
    
    @Override
    @Pure
    public int hashCode() {
      final int prime = 31;
      int result = 1;
      result = prime * result + ((this.beta0== null) ? 0 : this.beta0.hashCode());
      result = prime * result + ((this.beta1== null) ? 0 : this.beta1.hashCode());
      return prime * result + this.x;
    }
    
    @Override
    @Pure
    public boolean equals(final Object obj) {
      if (this == obj)
        return true;
      if (obj == null)
        return false;
      if (getClass() != obj.getClass())
        return false;
      Utils.Logistic other = (Utils.Logistic) obj;
      if (this.beta0 == null) {
        if (other.beta0 != null)
          return false;
      } else if (!this.beta0.equals(other.beta0))
        return false;
      if (this.beta1 == null) {
        if (other.beta1 != null)
          return false;
      } else if (!this.beta1.equals(other.beta1))
        return false;
      if (other.x != this.x)
        return false;
      return true;
    }
    
    @Pure
    public RealVar getBeta0() {
      return this.beta0;
    }
    
    @Pure
    public RealVar getBeta1() {
      return this.beta1;
    }
    
    @Pure
    public int getX() {
      return this.x;
    }
  }
  
  public static Map<String, RealVar> prediction(final Plate<String> rocketType, final Plate<Integer> launch, final Plated<RealVar> intercept, final Plated<RealVar> slope) {
    final LinkedHashMap<String, RealVar> it = new LinkedHashMap<String, RealVar>();
    Collection<Index<String>> _indices = rocketType.indices();
    for (final Index<String> rocket : _indices) {
      {
        final int x = launch.indices(rocket).size();
        final RealVar beta0 = intercept.get(rocket);
        final RealVar beta1 = slope.get(rocket);
        Utils.Logistic _logistic = new Utils.Logistic(beta0, beta1, x);
        it.put(rocket.key, _logistic);
      }
    }
    return it;
  }
}
